﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;

namespace s2t1
{
    class MainScene
    {
        public MainScene()
        {
            int success;
            var vert = GL.CreateShader(ShaderType.VertexShader);
            GL.ShaderSource(vert, Resource.shader_vert);
            GL.CompileShader(vert);
            GL.GetShader(vert, ShaderParameter.CompileStatus, out success);
            if (success == 0)
            {
                throw new Exception("vertex shader compile error:\n" + GL.GetShaderInfoLog(vert));
            }

            var frag = GL.CreateShader(ShaderType.FragmentShader);
            GL.ShaderSource(frag, Resource.shader_frag);
            GL.CompileShader(frag);
            GL.GetShader(frag, ShaderParameter.CompileStatus, out success);
            if (success == 0)
            {
                throw new Exception("fragment shader compile error:\n" + GL.GetShaderInfoLog(frag));
            }

            var program = GL.CreateProgram();
            GL.AttachShader(program, vert);
            GL.AttachShader(program, frag);
            GL.LinkProgram(program);
            GL.GetProgram(program, GetProgramParameterName.LinkStatus, out success);
            if (success == 0)
            {
                throw new Exception("shader link error:\n" + GL.GetProgramInfoLog(program));
            }
            _shader = program;

            tri1.Color = new Vector4(0, 0, 1, 1);
            tri1.AbsSpeed = new Vector2d(0.9, 0);
            tri1.InitialPosition = new Vector2d(0.0, 0.0);

            tri2.Color = new Vector4(1, 0, 0, 1);
            tri2.AbsSpeed = new Vector2d(-0.6, 0);
            tri2.InitialPosition = new Vector2d(0.8, 0.0);
        }

        public void Draw()
        {
            GL.UseProgram(_shader);
            GL.UniformMatrix4(GL.GetUniformLocation(_shader, "projection"), false, ref _projection);
            GL.UniformMatrix4(GL.GetUniformLocation(_shader, "view"), false, ref _view);

            Matrix4 model = Matrix4.Identity;

            tri1.Draw(_shader, model, 0, Vector2d.Zero);
            tri2.Draw(_shader, model, 0, Vector2d.Zero);
        }

        public void Update(double dt)
        {
            tri1.Update(dt, Vector2d.Zero);
            tri2.Update(dt, Vector2d.Zero);
        }

        private int _shader;

        private Matrix4 _projection = Matrix4.CreateOrthographicOffCenter(-1, 1, -1, 1, 20, -20);
        private Matrix4 _view = Matrix4.LookAt(Vector3.UnitZ, Vector3.Zero, Vector3.UnitX);

        private Triangle tri1 = new Triangle();
        private Triangle tri2 = new Triangle();
    }
}
